@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2007 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -14,7 +14,6 @@ COPYING distributed with the source files.
 #include <gtk/gtkprintjob.h>
 #endif
 
-#include <vector>
 #include <iomanip>
 #include <memory>
 #include <cstring>
@@ -59,8 +58,8 @@ COPYING distributed with the source files.
 #include "dialogs.h"
 #include "fax_list_icons.h"
 #include "utils/thread.h"
-#include "utils/shared_handle.h"
 #include "utils/utf8_utils.h"
+#include "utils/toolbar_append_widget.h"
 
 #ifdef HAVE_OSTREAM
 #include <ios>
@@ -77,6 +76,11 @@ COPYING distributed with the source files.
 #include <locale>
 #endif
 
+#if GTK_CHECK_VERSION(2,4,0)
+#include <gtk/gtktoolitem.h>
+#include <gtk/gtkseparatortoolitem.h>
+#endif
+
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
@@ -131,12 +135,14 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 			      working_dir(prog_config.working_dir),
                               fax_list_manager(mode) {
 
-  // we have included a std::string working_dir member so that we can access it in the
-  // FaxListDialog::print_fax_thread() and FaxListDialog::view_fax_thread() without
-  // having to introduce locking of accesses to prog_config.working_dir by the main
-  // GUI thread.  It is fine to initialise it in the initialisation list of this
-  // constructor as prog_config.working_dir is only set once, on the first call to
-  // configure_prog() (that is, when configure_prog() is passed false as its argument)
+  // we have included a std::string working_dir member so that we can
+  // access it in the FaxListDialog::print_fax_thread() without having
+  // to introduce locking of accesses to prog_config.working_dir by
+  // the main GUI thread.  It is fine to initialise it in the
+  // initialisation list of this constructor as
+  // prog_config.working_dir is only set once, on the first call to
+  // configure_prog() (that is, when configure_prog() is passed false
+  // as its argument)
 
   // notify the existence of this object
   if (mode == FaxListEnum::received) is_fax_received_list++;
@@ -188,8 +194,9 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   GtkWidget* image_p;
   image_p = gtk_image_new_from_stock(GTK_STOCK_PRINT, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(print_button_p), image_p);
-  gtk_toolbar_append_widget(toolbar_p, print_button_p,
-			    gettext("Print selected fax"), "");
+  toolbar_append_widget(toolbar_p, print_button_p,
+			gettext("Print selected fax"), "");
+
   gtk_widget_set_sensitive(print_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(print_button_p), GTK_RELIEF_NONE);
 
@@ -199,8 +206,8 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, view_button_p,
-			    gettext("View selected fax"), "");
+  toolbar_append_widget(toolbar_p, view_button_p,
+			gettext("View selected fax"), "");
   gtk_widget_set_sensitive(view_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(view_button_p), GTK_RELIEF_NONE);
 
@@ -210,22 +217,22 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(describe_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, describe_button_p,
-			    gettext("Add/amend description for selected fax"), "");
+  toolbar_append_widget(toolbar_p, describe_button_p,
+			gettext("Add/amend description for selected fax"), "");
   gtk_widget_set_sensitive(describe_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(describe_button_p), GTK_RELIEF_NONE);
 
   image_p = gtk_image_new_from_stock(GTK_STOCK_DELETE, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(delete_fax_button_p), image_p);
-  gtk_toolbar_append_widget(toolbar_p, delete_fax_button_p,
-			    gettext("Delete selected fax"), "");
+  toolbar_append_widget(toolbar_p, delete_fax_button_p,
+			gettext("Delete selected fax"), "");
   gtk_widget_set_sensitive(delete_fax_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(delete_fax_button_p), GTK_RELIEF_NONE);
 
   image_p = gtk_image_new_from_stock(GTK_STOCK_CLEAR, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(empty_trash_button_p), image_p);
-  gtk_toolbar_append_widget(toolbar_p, empty_trash_button_p,
-			    gettext("Empty trash folder"), "");
+  toolbar_append_widget(toolbar_p, empty_trash_button_p,
+			gettext("Empty trash folder"), "");
   gtk_widget_set_sensitive(empty_trash_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(empty_trash_button_p), GTK_RELIEF_NONE);
 
@@ -235,8 +242,8 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(add_folder_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, add_folder_button_p,
-			    gettext("Add new folder"), "");
+  toolbar_append_widget(toolbar_p, add_folder_button_p,
+			gettext("Add new folder"), "");
   gtk_widget_set_sensitive(add_folder_button_p, true);
   gtk_button_set_relief(GTK_BUTTON(add_folder_button_p), GTK_RELIEF_NORMAL);
 
@@ -246,8 +253,8 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(delete_folder_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, delete_folder_button_p,
-			    gettext("Delete selected folder"), "");
+  toolbar_append_widget(toolbar_p, delete_folder_button_p,
+			gettext("Delete selected folder"), "");
   gtk_widget_set_sensitive(delete_folder_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(delete_folder_button_p), GTK_RELIEF_NONE);
 
@@ -293,6 +300,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   GtkWidget* toolbar_box_p = gtk_hbox_new(false, 0);
   gtk_box_pack_start(GTK_BOX(toolbar_box_p), GTK_WIDGET(toolbar_p), true, true, 0);
 
+  GtkWidget* new_fax_frame_p = 0;
   // if mode == FaxListEnum::sent we don't do anything at the moment
   if (mode == FaxListEnum::received) {
 
@@ -300,30 +308,33 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     reset_button_p = gtk_button_new();
     image_p = gtk_image_new_from_stock(GTK_STOCK_UNDO, efax_gtk_button_size);
     gtk_container_add(GTK_CONTAINER(reset_button_p), image_p);
-    gtk_toolbar_append_widget(toolbar_p, reset_button_p,
-			      gettext("Reset new faxes count"), "");
+    toolbar_append_widget(toolbar_p, reset_button_p,
+			  gettext("Reset new faxes count"), "");
     gtk_widget_set_sensitive(reset_button_p, true);
     gtk_button_set_relief(GTK_BUTTON(reset_button_p), GTK_RELIEF_NORMAL);
     g_signal_connect(G_OBJECT(reset_button_p), "clicked",
 		     G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
 
-    // this is to put a label at the far right of the toolbar to give the number
-    // of received faxes since the fax list was last closed
-    // it would be easier to do this using GtkSeparatorToolItem but that
-    // is only available with GTK+ 2.4 onwards and I want to keep compatibility
-    // with earlier versions, so make a new toolbar for the purpose and put the
-    // toolbars in a GtkHBox object
-    GtkWidget* new_fax_toolbar_p = gtk_toolbar_new();
-    gtk_box_pack_end(GTK_BOX(toolbar_box_p), new_fax_toolbar_p, false, false, 0);
-
     // now provide a display for the new received fax count
-    GtkWidget* new_fax_frame_p = gtk_frame_new(0);
+    new_fax_frame_p = gtk_frame_new(0);
     gtk_container_set_border_width(GTK_CONTAINER(new_fax_frame_p), 1);
-    gtk_toolbar_append_widget(GTK_TOOLBAR(new_fax_toolbar_p), new_fax_frame_p, "", "");
     new_fax_label_p = gtk_label_new(0);
     gtk_misc_set_padding(GTK_MISC(new_fax_label_p), 6, 2);
     gtk_container_add(GTK_CONTAINER(new_fax_frame_p), new_fax_label_p);
     gtk_label_set_justify(GTK_LABEL(new_fax_label_p), GTK_JUSTIFY_RIGHT);
+
+#if GTK_CHECK_VERSION(2,4,0)
+    GtkToolItem* separator_p = gtk_separator_tool_item_new();
+    gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(separator_p), false);
+    gtk_tool_item_set_expand(separator_p, true);
+    gtk_toolbar_insert(GTK_TOOLBAR(toolbar_p), separator_p, -1);
+    toolbar_append_widget(GTK_TOOLBAR(toolbar_p), new_fax_frame_p, "", "");
+#else
+    GtkWidget* new_fax_toolbar_p = gtk_toolbar_new();
+    gtk_box_pack_end(GTK_BOX(toolbar_box_p), new_fax_toolbar_p, false, false, 0);
+    toolbar_append_widget(GTK_TOOLBAR(new_fax_toolbar_p), new_fax_frame_p, "", "");
+#endif
+
     // as this is in the constructor, get_new_fax_count() always shows a 0 count
     // but this is OK as we are using it here to size the label - the user must
     // call display_new_fax_count() again once get_new_fax_count_sig has been
@@ -353,10 +364,39 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(fax_list_box_p));
 
   gtk_window_set_default_size(get_win(), standard_size * 18, standard_size * 16);
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar_p), false);
+#endif
   gtk_widget_show_all(GTK_WIDGET(get_win()));
+
+  // not needed if gtk_toolbar_set_show_arrow() has been set false, or GTK+ version < 2.4
+  /*
+  if (mode == FaxListEnum::received) {
+    gtk_widget_set_size_request(GTK_WIDGET(toolbar_p),
+				print_button_p->allocation.width + view_button_p->allocation.width
+				+ describe_button_p->allocation.width + delete_fax_button_p->allocation.width
+				+ empty_trash_button_p->allocation.width + add_folder_button_p->allocation.width
+				+ delete_folder_button_p->allocation.width + reset_button_p->allocation.width
+				+ new_fax_frame_p->allocation.width + 16,
+				print_button_p->allocation.height);
+  }
+  else {
+    gtk_widget_set_size_request(GTK_WIDGET(toolbar_p),
+				print_button_p->allocation.width + view_button_p->allocation.width
+				+ describe_button_p->allocation.width + delete_fax_button_p->allocation.width
+				+ empty_trash_button_p->allocation.width + add_folder_button_p->allocation.width
+				+ delete_folder_button_p->allocation.width + 12,
+				print_button_p->allocation.height);
+  }
+  */
 }
 
 FaxListDialog::~FaxListDialog(void) {
+
+  std::vector<SharedHandle<char*> >::const_iterator iter;
+  for (iter = view_files.begin(); iter != view_files.end(); ++iter) {
+    unlink(iter->get());
+  }
   // notify the destruction of this object
   if (mode == FaxListEnum::received) is_fax_received_list--;
   else is_fax_sent_list--;
@@ -1130,12 +1170,15 @@ void FaxListDialog::print_fax_thread(std::string* arg_p, bool use_gtkprint) {
 
 void FaxListDialog::view_fax(void) {
  
-  // we do not need a mutex before reading prog_config.ps_view_cmd
-  // as although read in FaxListDialog::get_ps_viewer_parms(),
-  // the thread which executes that method is launched below as
-  // FaxListsDialog::view_fax_thread() and is therefore not running
-  // at this stage
-  if (fax_list_manager.is_fax_selected() == 1 && !prog_config.ps_view_cmd.empty()) {
+  bool have_view_cmd;
+  // lock the Prog_config object to stop it being modified or accessed in the initial (GUI)
+  // thread while we are accessing it here
+  {
+    Thread::Mutex::Lock lock(*prog_config.mutex_p);
+    have_view_cmd = !prog_config.ps_view_cmd.empty();
+  }
+
+  if (fax_list_manager.is_fax_selected() == 1 && have_view_cmd) {
 
     std::auto_ptr<std::string> fax_basename_a(new std::string(prog_config.working_dir));
     if (mode == FaxListEnum::received) *fax_basename_a += "/faxin/";
@@ -1153,6 +1196,23 @@ void FaxListDialog::view_fax(void) {
     *fax_basename_a += fax_number;
     *fax_basename_a += '.';
 
+    // get a temporary file
+    std::auto_ptr<std::string> filename_a(new std::string(prog_config.working_dir));
+    *filename_a += "/efax-gtk-view-XXXXXX";
+
+    std::string::size_type size = filename_a->size() + 1;
+    SharedHandle<char*> tempfile_h(new char[size]);
+    std::memcpy(tempfile_h.get(), filename_a->c_str(), size); // this will include the terminating '\0' in the copy
+    int file_fd = mkstemp(tempfile_h.get());
+
+    if (file_fd == -1) {
+      write_error("Failed to make temporary file for viewing");
+      return;
+    }
+
+    view_files.push_back(tempfile_h);
+    *filename_a = tempfile_h.get();
+
     // now block off the signals for which we have set handlers so that the socket server
     // thread does not receive the signals, otherwise we will have memory synchronisation
     // issues in multi-processor systems - we will unblock in the initial (GUI) thread
@@ -1170,12 +1230,18 @@ void FaxListDialog::view_fax(void) {
     // in the event of memory allocation failure but as a matter of policy we
     // do not deal with that condition in this program so we can pass a raw pointer
     // rather than the std::auto_ptr<> object
-    std::string* arg_p = fax_basename_a.release();
+    std::string* fax_basename_p = fax_basename_a.release();
+    std::pair<std::string*, int> file_pair;
+    file_pair.first = filename_a.release();
+    file_pair.second = file_fd;
     std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(sigc::bind(sigc::mem_fun(*this, &FaxListDialog::view_fax_thread), arg_p),
+      Thread::Thread::start(sigc::bind(sigc::mem_fun(*this, &FaxListDialog::view_fax_thread),
+				       fax_basename_p, file_pair),
 			    false);
     if (!res.get()) {
-      delete arg_p;
+      delete fax_basename_p;
+      delete file_pair.first;
+      while (::close(file_fd) == -1 && errno == EINTR);
       write_error("Cannot start new view fax thread, fax cannot be viewed\n");
     }
     // now unblock the signals so that the initial (GUI) thread can receive them
@@ -1183,27 +1249,11 @@ void FaxListDialog::view_fax(void) {
   }
 }
 
-void FaxListDialog::view_fax_thread(std::string* arg_p) {
+void FaxListDialog::view_fax_thread(std::string* fax_basename_p,
+				    std::pair<std::string*, int> file_pair) {
  
-  const std::auto_ptr<std::string> fax_basename_a(arg_p);
-
-  // get a temporary file
-  std::string filename;
-  {
-    Thread::Mutex::Lock lock(working_dir_mutex);
-    filename = working_dir;
-  }
-  filename += "/efax-gtk-view-XXXXXX";
-
-  std::string::size_type size = filename.size() + 1;
-  ScopedHandle<char*> tempfile_h(new char[size]);
-  std::memcpy(tempfile_h.get(), filename.c_str(), size); // this will include the terminating '\0' in the copy
-  int file_fd = mkstemp(tempfile_h.get());
-
-  if (file_fd == -1) {
-    write_error("Failed to make temporary file for viewing");
-    return;
-  }
+  const std::auto_ptr<std::string> fax_basename_a(fax_basename_p);
+  const std::auto_ptr<std::string> filename_a(file_pair.first);
 
   // now create a pipe and proceed to fork to write the postscript to temporary file
   PipeFifo fork_pipe(PipeFifo::block);
@@ -1237,9 +1287,8 @@ void FaxListDialog::view_fax_thread(std::string* arg_p) {
 		"and the PATH environmental variable\n");
     _exit(0);
   }
-  // this is the parent process, which will convert the fax to Postscript
-  // and then pipe it on stdout to write_to_file()
-    
+  // this is the parent process,
+
   // first, release the memory allocated on the heap for
   // the redundant fax_to_ps_parms
   // we are in the main parent process here - no worries about
@@ -1248,8 +1297,8 @@ void FaxListDialog::view_fax_thread(std::string* arg_p) {
     
   // now write from the pipe (containing the generated PS text) to file
   fork_pipe.make_readonly();
-  write_pipe_to_file(&fork_pipe, file_fd);
-  while (::close(file_fd) == -1 && errno == EINTR);
+  write_pipe_to_file(&fork_pipe, file_pair.second);
+  while (::close(file_pair.second) == -1 && errno == EINTR);
 
   // the temporary file to be viewed has now been created
   // so launch a new process to control the viewing process
@@ -1260,7 +1309,7 @@ void FaxListDialog::view_fax_thread(std::string* arg_p) {
   // viewer following the next fork() call (because this is a multi-
   // threaded program, we must do this before fork()ing because we use
   // functions to get the arguments which are not async-signal-safe)
-  std::pair<const char*, char* const*> ps_viewer_parms(get_ps_viewer_parms(tempfile_h.get()));
+  std::pair<const char*, char* const*> ps_viewer_parms(get_ps_viewer_parms(filename_a->c_str()));
 
   if (ps_viewer_parms.first) { // this will be 0 if get_ps_viewer_parms()
                                // threw a Utf8::ConversionError)
@@ -1292,36 +1341,12 @@ void FaxListDialog::view_fax_thread(std::string* arg_p) {
 
       connect_to_stderr();
 
-      // now fork() again
-      pid_t pid = fork();
+      execvp(ps_viewer_parms.first, ps_viewer_parms.second);
 
-      if (pid == -1) {
-	write_error("Fork error - exiting\n");
-	_exit(FORK_ERROR);
-      }
-    
-      if (!pid) { // child process to call ps_viewer() from
-	  
-	execvp(ps_viewer_parms.first, ps_viewer_parms.second);
-
-	// if we reached this point, then the execvp() call must have failed
-	// report error and then end process - use _exit(), not exit()
-	write_error("Can't find the ps viewer program - please check your installation\n"
-		    "and the PATH environmental variable\n");
-	_exit(0);
-      }
-      // now we are back to the main viewing process again
-      // delete the temporary file
-
-      // first wait till the ps_viewer() process is closed by the user exiting
-      // the postscript viewing program
-      // Note that we have already fork()ed so this process will not be
-      // receiving signals so we do not need to test for EINTR on the
-      // call to wait()
-      wait(0);
-    
-      unlink(tempfile_h.get());
-      // now terminate the process
+      // if we reached this point, then the execvp() call must have failed
+      // report error and then end process - use _exit(), not exit()
+      write_error("Can't find the ps viewer program - please check your installation\n"
+		  "and the PATH environmental variable\n");
       _exit(0);
     }
     // release the memory allocated on the heap for
