@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2007 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -23,6 +23,7 @@ COPYING distributed with the source files.
 
 #include <string>
 #include <utility>
+#include <vector>
 
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkwindow.h>
@@ -33,6 +34,7 @@ COPYING distributed with the source files.
 #include "utils/window.h"
 #include "utils/pipes.h"
 #include "utils/mutex.h"
+#include "utils/shared_handle.h"
 
 namespace { // we put the function in anonymous namespace in fax_list.cpp
             // so it is not exported at link time
@@ -50,6 +52,8 @@ class FaxListDialog: public sigc::trackable, public WinBase {
   Thread::Mutex working_dir_mutex;
   std::string working_dir;
 
+  std::vector<SharedHandle<char*> > view_files;
+
   FaxListManager fax_list_manager;
 
 #if GTK_CHECK_VERSION(2,10,0)
@@ -83,7 +87,7 @@ class FaxListDialog: public sigc::trackable, public WinBase {
   void print_fax(void);
   void print_fax_thread(std::string*, bool);
   void view_fax(void);
-  void view_fax_thread(std::string*);
+  void view_fax_thread(std::string*, std::pair<std::string*, int>);
   void delete_parms(std::pair<const char*, char* const*>);
 public:
   friend void FaxListDialogCB::fax_list_button_clicked(GtkWidget*, void*);
