@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2007 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -68,6 +68,7 @@ COPYING distributed with the source files.
 #include <gtk/gtkimage.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkiconfactory.h>
+#include <gtk/gtksettings.h>
 #include <gtk/gtkenums.h>
 #include <gtk/gtkmain.h>
 
@@ -691,6 +692,15 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
 
   get_window_settings();
 		    
+  // don't have entry widgets selecting on focus
+  GtkSettings* settings_p = gtk_settings_get_default();
+  if (settings_p) {
+    g_object_set(settings_p,
+		 "gtk-entry-select-on-focus", false,
+		 static_cast<void*>(0));
+  }
+  else write_error("Can't obtain default GtkSettings object\n");
+
   if (filename) {
     // this will also call set_file_items_sensitive_impl() if the call to
     // get_window_settings() has set socket_button active
@@ -1406,7 +1416,7 @@ void MainWindow::about_impl(bool efax_gtk) {
 
   if (efax_gtk) {
     std::string message("efax-gtk-" VERSION "\n");
-    message += gettext("Copyright (C) 2001 - 2006 Chris Vine\n\n"
+    message += gettext("Copyright (C) 2001 - 2007 Chris Vine\n\n"
 		       "This program is released under the "
 		       "GNU General Public License, version 2");
     dialog_p = new InfoDialog(message.c_str(), gettext("About efax-gtk"),
