@@ -28,6 +28,10 @@ typedef signed int ssize_t;
 typedef unsigned int socklen_t;
 #endif
 
+#ifndef HAVE_IN_ADDR_T
+typedef unsigned long in_addr_t;
+#endif
+
 // deal with libsigc++
 #if SIGC_VERSION < 20
 #include "sigc_compatibility.h"
@@ -73,8 +77,9 @@ using namespace std;    // until I get a Std C++ compliant compiler to test with
      localtime() - used in the main (GUI) thread (efax_controller.cpp)
      readdir()   - used in the main (GUI) thread (fax_list.cpp)
 
-     gethostbyaddr() ) used in the socket server thread (socket_server.cpp)
-     gethostbyname() )
+     gethostbyaddr() )
+     gethostbyname() ) used in the socket server thread (socket_server.cpp)
+     inet_ntoa()     )
 
    If the program is modified to use any of these in a different thread,
    mutexes will need to be applied.
@@ -110,20 +115,24 @@ using namespace std;    // until I get a Std C++ compliant compiler to test with
    - a temporary thread to prepare a fax for printing, involving the
      following methods of the FaxListDialog class:
 
-       print_fax_thread(), get_print_from_stdin_parms(),
-       get_fax_to_ps_parms() and delete_parms() (get_fax_to_ps_parms()
+       print_fax_thread(), write_pipe_to_file(),
+       get_print_from_stdin_parms(), get_fax_to_ps_parms() and
+       delete_parms() (write_pipe_to_file() and get_fax_to_ps_parms()
        is also called by the temporary thread invoked with
        view_fax_thread() and delete_parms() is also called by that
        thread and by the main (GUI) thread)
 
+     together with PrintManager::set_filename() and the invocation of
+     the PrintManager::print Notifier object.
+
    - a temporary thread to view a fax, involving the following methods
      of the FaxListDialog class:
 
-       view_fax_thread(), write_from_stdin(), get_ps_viewer_parms(),
-       get_fax_to_ps_parms() and delete_parms() (get_fax_to_ps_parms()
-       is also called by the temporary thread invoked with
-       print_fax_thread() and delete_parms() is also called by that
-       thread and by the main (GUI) thread)
+       view_fax_thread(), write_pipe_to_file(), get_ps_viewer_parms(),
+       get_fax_to_ps_parms() and delete_parms() (write_pipe_to_file()
+       and get_fax_to_ps_parms() is also called by the temporary
+       thread invoked with print_fax_thread() and delete_parms() is
+       also called by that thread and by the main (GUI) thread)
 
 3. The last member of the Prog_config class below is a pointer to a mutex object
    allocated in int main() in main.cpp.  The mutex should be locked whenever:
@@ -207,6 +216,7 @@ struct Prog_config {
   bool tone_dial;
   bool found_rcfile;
   bool GPL_flag;
+  bool gtkprint;
   bool print_popup;
   bool sock_server;
   bool sock_popup;
