@@ -23,6 +23,7 @@ COPYING distributed with the source files.
 #include "settings_help.h"
 #include "utils/window.h"
 #include "utils/widget.h"
+#include "utils/gobj_handle.h"
 #include "utils/utf8_utils.h"
 
 
@@ -140,22 +141,31 @@ namespace PrintPageCB {
 class PrintPage: public MainWidgetBase {
   PrintMessages help_messages;
 
+  GtkWidget* command_label_p;
+  GtkWidget* popup_label_p;
+  GobjHandle<GtkWidget> gtkprint_check_button_h;
   GtkWidget* command_entry_p;
   GtkWidget* shrink_spin_button_p;
   GtkWidget* popup_check_button_p;
+  GobjHandle<GtkWidget> gtkprint_help_button_h;
   GtkWidget* command_help_button_p;
   GtkWidget* shrink_help_button_p;
   GtkWidget* popup_help_button_p;
+
+  void gtkprint_button_toggled_impl(void);
 public:
   friend void PrintPageCB::print_page_button_clicked(GtkWidget*, void*);
 
   sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  std::string get_gtkprint(void) const;
   std::string get_command(void) const;
   std::string get_shrink(void) const;
   std::string get_popup(void) const;
+  void set_gtkprint(const std::string& gtkprint_string);
   void set_command(const std::string& command);
   void set_shrink(const std::string& shrink);
   void set_popup(const std::string& popup_string);
+
   void clear(void);
   PrintPage(const int);
 };
